from django.db import models

# Create your models here.

class Bloc(models.Model):
    ROSA = 'R'
    GROC = 'G'
    TARONJA ='T'
    VERMELL = 'V'
    MARRO = 'M'
    VERD = 'VE'
    BLAU = 'B'
    LILA = 'L'
    NEGRE = 'N'

    TAULA_COLORS = (
        (ROSA, 'rosa'),
        (GROC, 'groc'),
        (TARONJA, 'taronja'),
        (VERMELL, 'vermell'),
        (MARRO, 'marro'),
        (VERD, 'verd'),
        (BLAU, 'blau'),
        (LILA, 'lila'),
        (NEGRE, 'negre'),
    )

    zona = models.ForeignKey('Zona')
    nom = models.CharField(max_length=20, blank=False, null=False)
    grau = models.CharField(max_length=2, choices=TAULA_COLORS, default=ROSA)
    data_creacio = models.DateTimeField(auto_now_add=True)
    actiu = models.BooleanField(default=True)

    punts =  models.IntegerField()

    def save(self, *args, **kwargs):
        puntuacio = {'R':40, 'G':50, 'T':60, 'V':80, 'M':100, 'VE':120, 'B':160, 'L':200, 'N':240}
        punts = puntuacio[self.grau]

        self.punts = punts
        super(Bloc, self).save(*args, **kwargs)


class Encadena(models.Model):
    soci = models.ForeignKey('Soci')
    bloc = models.ForeignKey('Bloc')
    n_intents = models.IntegerField()
    data_encadene = models.DateTimeField(auto_now_add=True)
    comentaris = models.CharField(max_length=150, blank=True, null=True)
    punts =  models.IntegerField()

    class Meta:
        unique_together = ['soci', 'bloc']

    def save(self, *args, **kwargs):
        intents = self.n_intents
        if intents > 3: intents = 4
        punts = self.bloc.punts -(1 -intents) *10

        self.punts = punts
        super(Encadena, self).save(*args, **kwargs)


class Soci(models.Model):
    user = models.ForeignKey('auth.User', related_name='s_user')
    nom = models.CharField(max_length=20, blank=False, null=False)
    cognom = models.CharField(max_length=20, blank=False, null=False)
    cognom2 = models.CharField(max_length=20, blank=False, null=False)
    email = models.CharField(max_length=20, blank=False, null=False)
    telefon = models.CharField(max_length=20, blank=False, null=False)
    data_naixement = models.DateField()
    data_inscripcio = models.DateTimeField(auto_now_add=True)

    img_perfil = models.ImageField(upload_to='profile', default=None)

    def __unicode__(self):
	    return '%s %s' % (self.nom, self.cognom)


class Zona(models.Model):
    nom = models.CharField(max_length=20, blank=False, null=False)
    ubicacio = models.CharField(max_length=20, blank=False, null=False)


class Favorits(models.Model):
    soci = models.ForeignKey('Soci', related_name='f_soci')
    favorits = models.ManyToManyField(Soci, related_name='f_favorits')
