from rest_framework import serializers
from django.db.models import Sum

from ranking.models import *


# Serializers define the API representation.

class SociSerializer(serializers.ModelSerializer):
    puntuacio = serializers.SerializerMethodField('getPunts')
    
    def getPunts(self, instance):
        puntuacio = Encadena.objects.filter(soci=instance).aggregate(Sum('punts')).get('punts__sum',0)
        if not puntuacio: puntuacio = 0
        return puntuacio

    class Meta:
        model = Soci
        fields = ('pk', 'nom', 'cognom', 'img_perfil', 'puntuacio')


class SociByBlocSerializer(serializers.ModelSerializer):
    pk = serializers.SerializerMethodField('getPkSoci')
    nom = serializers.SerializerMethodField('getNomSoci')
    cognom = serializers.SerializerMethodField('getCognomSoci')
    puntuacio = serializers.SerializerMethodField('getPuntsSoci')

    def getPkSoci(self, instance):
        return instance.soci.pk

    def getNomSoci(self, instance):
        return instance.soci.nom

    def getCognomSoci(self, instance):
        return instance.soci.cognom

    def getPuntsSoci(self, instance):
        puntuacio = Encadena.objects.filter(soci=instance.soci).aggregate(Sum('punts')).get('punts__sum',0)
        if not puntuacio: puntuacio = 0
        return puntuacio

    class Meta:
        model = Soci
        fields = ('pk', 'nom', 'cognom', 'puntuacio')


class SociPkSerializer(serializers.ModelSerializer):
    class Meta:
        model = Soci
        fields = ('pk', )


class BlocSerializer(serializers.ModelSerializer):
    class Meta:
        model = Bloc
        fields = ('pk', 'nom', 'zona', 'grau', 'actiu', 'punts')


class EncadenaSerializer(serializers.ModelSerializer):
    bloc_nom = serializers.SerializerMethodField('getNomBloc')
    bloc_grau = serializers.SerializerMethodField('getGrauBloc')

    def getNomBloc(self, instance):
        return instance.bloc.nom

    def getGrauBloc(self, instance):
        return instance.bloc.grau

    class Meta:
        model = Encadena
        fields = ('bloc_nom', 'bloc_grau', 'n_intents', 'data_encadene')


class BlocBySociSerializer(serializers.ModelSerializer):
    nom = serializers.SerializerMethodField('getNomBloc')
    zona = serializers.SerializerMethodField('getZonaBloc')
    grau = serializers.SerializerMethodField('getGrauBloc')
    actiu = serializers.SerializerMethodField('getActiuBloc')
    punts = serializers.SerializerMethodField('getPuntsBloc')
    
    def getNomBloc(self, instance):
        return instance.bloc.nom

    def getZonaBloc(self, instance):
        return instance.bloc.zona.pk

    def getGrauBloc(self, instance):
        return instance.bloc.grau

    def getActiuBloc(self, instance):
        return instance.bloc.actiu

    def getPuntsBloc(self, instance):
        return instance.bloc.punts

    class Meta:
        model = Bloc
        fields = ('pk', 'nom', 'zona', 'grau', 'actiu', 'punts')


class ComentarisSerializer(serializers.ModelSerializer):
    nom_soci = serializers.SerializerMethodField('getNomSoci')
    cognom_soci = serializers.SerializerMethodField('getCognomSoci')

    def getNomSoci(self, instance):
        return instance.soci.nom

    def getCognomSoci(self, instance):
        return instance.soci.cognom

    class Meta:
        model = Encadena
        fields = ('nom_soci', 'cognom_soci', 'comentaris', 'data_encadene')


class EncadenaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Encadena
        fields = ('soci', 'bloc', 'n_intents', 'comentaris')


class FavoritsSerializer(serializers.ModelSerializer):
    class Meta:
        model = Favorits
        fields = ('soci', 'favorits')


class UploadPasswdSerializer(serializers.ModelSerializer):
    class Meta:
        model = Soci
        fields = ('pk', 'passwd')
