from django.views.generic.detail import DetailView
from django.views.generic.list import ListView
from django.shortcuts import render
from django.http import JsonResponse
from django.http import HttpResponse

from django.template import RequestContext, loader
from django.db.models import Sum

from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response

from rest_framework import status
from rest_framework.response import Response
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.authentication import SessionAuthentication, BasicAuthentication, TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from forms import UploadFileForm

from ranking.serializers import *

import json

# Create your views here.

from .models import Soci
from .models import Bloc
from .models import Encadena

####################
# auth zone: vista que crea un token automaticament al donar d'alta un nou usuari.
####################

from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver

from rest_framework.authtoken.models import Token

@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def create_auth_token(sender, instance=None, created=False, **kwargs):
    if created:
        Token.objects.create(user=instance)

####################
# end auth zone
####################

# funcio que retorna el top total de socis
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def top(request):
    top_list = Soci.objects.all().annotate(punts=Sum('encadena__bloc__punts')).order_by('-punts')
    top_list_serialized = SociSerializer(top_list, many=True)

    return Response(top_list_serialized.data)


# funcio que retorna el top n de socis
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def top_n(request, n):
    top_n_list = Soci.objects.all().annotate(punts=Sum('encadena__bloc__punts')).order_by('-punts')[:n]
    top_n_list_serialized = SociSerializer(top_n_list, many=True)

    return Response(top_n_list_serialized.data)


# funcio que retorna el llistat de socis
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def soci(request):
    soci_list = Soci.objects.all()
    soci_serialized = SociSerializer(soci_list, many=True)

    return Response(soci_serialized.data)


# funcio que retorna les dades de un soci n
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def soci_n(request, id_soci):
    soci_n_list = Soci.objects.filter(pk=id_soci)
    soci_n_serialized = SociSerializer(soci_n_list, many=True)

    return Response(soci_n_serialized.data)


# funcio que retorna els socis que han encadenat un bloc en format "Soci"
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def soci_sn(request, id_bloc):
    soci_sn_list = Encadena.objects.filter(bloc__pk=id_bloc).order_by('-data_encadene')
    soci_sn_serialized = SociByBlocSerializer(soci_sn_list, many=True)

    return Response(soci_sn_serialized.data)


# funcio que retorna la pk de un soci a partir de el seu token 
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def soci_pk(request):
    soci_pk_list = Soci.objects.filter(user=request.user)
    soci_pk_serialized = SociPkSerializer(soci_pk_list, many=True)

    return Response(soci_pk_serialized.data)


# funcio que retorna el llistat de blocs
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def bloc(request):
    bloc_list = Bloc.objects.all().order_by('-actiu')
    bloc_serialized = BlocSerializer(bloc_list, many=True)

    return Response(bloc_serialized.data)


# funcio que retorna les dades de un bloc n
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def bloc_n(request, id_bloc):
    bloc_n_list = Bloc.objects.filter(pk=id_bloc)
    bloc_n_serialized = BlocSerializer(bloc_n_list, many=True)

    return Response(bloc_n_serialized.data)


# funcio que retorna els blocs que encadena un soci amb la puntuacio obtingda
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def bloc_en(request, id_soci):
    bloc_e_list = Encadena.objects.filter(soci=id_soci).order_by('-data_encadene')
    bloc_e_serialized = EncadenaSerializer(bloc_e_list, many=True)

    return Response(bloc_e_serialized.data)


# funcio que retorna els blocs que encadena un soci en format "Bloc"
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def bloc_sn(request, id_soci):
    bloc_sn_list = Encadena.objects.filter(soci=id_soci).order_by('-data_encadene')
    bloc_sn_serialized = BlocBySociSerializer(bloc_sn_list, many=True)

    return Response(bloc_sn_serialized.data)


# funcio que retorna els comentris de un bloc
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def comentaris(request, id_bloc):
    comentaris_list = Encadena.objects.filter(bloc=id_bloc).order_by('-data_encadene')
    comentaris_serialized = ComentarisSerializer(comentaris_list, many=True)

    return Response(comentaris_serialized.data)


# funcio que registra un nou "encadene"
@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def encadena(request):
    serializer = EncadenaSerializer(favorits, data=request.data)

    if serializer.is_valid():
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# funcio que retorna els socis favorits del soci id_soci
@api_view(['GET'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def favorits(request, id_soci):
    favorit_obj = Favorits.objects.get(soci=id_soci)

    soci_list = favorit_obj.favorits.all()
    soci_serialized = SociSerializer(soci_list, many=True)

    return Response(soci_serialized.data)


# funcio que permet afegir un favorit
@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def add_favorits(request):
    serializer = FavoritsSerializer(data=request.data)

    if serializer.is_valid():
        soci = int(serializer.data['soci'])
        favorit = int(serializer.data['favorits'][0])

        Favorits.objects.get(soci=soci).favorits.add(favorit)

        return Response(serializer.data, status=status.HTTP_200_OK)

    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# funcio que permet eliminar un favorit
@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def del_favorits(request):
    serializer = FavoritsSerializer(data=request.data)

    if serializer.is_valid():
        soci = int(serializer.data['soci'])
        favorit = int(serializer.data['favorits'][0])

        Favorits.objects.get(soci=soci).favorits.remove(favorit)

        return Response(serializer.data, status=status.HTTP_200_OK)

    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


# funcio que permet pujar una nova imatge
@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def upload_image(request, id_soci):
    if request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)

        if form.is_valid():
            new_file = Soci( file = request.FILES[ 'file' ] )
            new_file.save()

            return HttpResponseRedirect( reverse( 'main:home' ) )
    else:
        form = UploadFileForm()

    data = {'form': form}
    return render_to_response( 'main/index.html', data, context_instance=RequestContext(request ))


# funcio que permet canviar el passwd de un usuari
@api_view(['POST'])
@authentication_classes((SessionAuthentication, BasicAuthentication, TokenAuthentication))
@permission_classes((IsAuthenticated,))
def upload_passwd(request):
    serializer = UploadPasswdSerializer(data=request.data)

    if serializer.is_valid():
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

