package ub.mywall;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

public class BlocArrayAdapter extends ArrayAdapter<Bloc>
{
    private List<Bloc> blocList;

    private Context context;

    public void updateData( ArrayList<Bloc> blocList )
    {
        this.blocList = blocList;
        this.notifyDataSetChanged();
    }

    static class BlocViewHolder
    {
        ImageView grau;

        TextView nom_bloc;
        TextView punts;

    }

    public BlocArrayAdapter(Context context, int textViewResourceId, ArrayList<Bloc> blocList )
    {
        super( context, textViewResourceId );

        this.context = context;
        this.blocList = blocList;
    }

    @Override
    public int getCount()
    {
        if ( this.blocList != null )
            return this.blocList.size();

        else return 0;
    }

    @Override
    public Bloc getItem ( int index ) {
        return this.blocList.get( index );
    }

    @Override
    public View getView( int position, View convertView, ViewGroup parent )
    {
        View row = convertView;
        BlocViewHolder viewHolder;

        final Bloc bloc = getItem( position );

        if ( row == null )
        {
            LayoutInflater inflater = ( LayoutInflater ) this.getContext().getSystemService( Context.LAYOUT_INFLATER_SERVICE );

            if ( bloc.getActiu().equals( "true" ) )
            {
                row = inflater.inflate(R.layout.listview_row_layout_blocs, parent, false);

            } else {
                row = inflater.inflate(R.layout.listview_row_layout_blocs_disabled, parent, false);
            }

            viewHolder = new BlocViewHolder();

            viewHolder.grau = ( ImageView ) row.findViewById( R.id.grau );
            viewHolder.nom_bloc = ( TextView ) row.findViewById( R.id.nom_bloc );
            viewHolder.punts = ( TextView ) row.findViewById( R.id.punts);

            row.setTag(viewHolder);

        } else {
            viewHolder = ( BlocViewHolder ) row.getTag();
        }

        // Log.i ("BlocArrayAdapter","getView - bloc.getNom():" +bloc.getNom() );

        try {
            viewHolder.grau.setBackgroundColor( bloc.getColorGrau() );

        } catch (Exception e) {
            Log.e("BlocArrayAdapter","getView - error en bloc.getGrau():" +bloc.getGrau() );
        }

        viewHolder.nom_bloc.setText( bloc.getNom() );
        viewHolder.punts.setText( bloc.getPunts() +" punts" );

        row.setOnClickListener( new View.OnClickListener()
        {
            @Override
            public void onClick( View v )
            {
                Intent i = new Intent( context, BlocOnClick.class );

                // sending data to new activity
                i.putExtra( "pk_bloc", bloc.getPk() );
                i.putExtra( "nom_bloc", bloc.getNom() );
                i.putExtra( "zona", bloc.getZona() );
                i.putExtra( "actiu", bloc.getActiu() );
                i.putExtra( "grau_bloc", bloc.getGrau() );

                context.startActivity(i);
            }
        });

        return row;

    }

    public Bitmap decodeToBitmap(byte[] decodedByte) {
        return BitmapFactory.decodeByteArray( decodedByte, 0, decodedByte.length );

    }

}