package ub.mywall;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONObject;

import java.io.InputStream;


public class BlocOnClick extends Activity
{
    private Context context;

    private String pk_bloc;
    private String pk_soci;
    private String token;

    private String nom_bloc;
    private String zona;
    private String actiu;
    private String grau_bloc;

    private EditText n_intents;
    private EditText comentaris;

    private String strN_intents, strComentaris;

    private RelativeLayout relativeLayout;

    private Boolean CS;

    static class BlocViewHolder
    {
        ImageView grau_bloc;

        TextView nom_bloc;
        TextView zona;
    }

    @Override
    public void onCreate( Bundle savedInstanceState )
    {
        super.onCreate( savedInstanceState );
        this.setContentView( R.layout.activity_bloc);

        relativeLayout = ( RelativeLayout ) findViewById( R.id.post_encadena );

        Intent i = getIntent();

        // boolea per a controlar is estem mostrant socis que encadenen bloc o comentaris
        CS = true;

        SharedPreferences mShared = this.getSharedPreferences( "claus", Context.MODE_PRIVATE );
        pk_soci = mShared.getString("pk_soci", null);
        token = mShared.getString( "token", null );

        // getting attached intent data
        this.pk_bloc = i.getStringExtra("pk_bloc");
        this.nom_bloc = i.getStringExtra("nom_bloc");
        this.zona = i.getStringExtra("zona");
        this.actiu = i.getStringExtra("actiu");
        this.grau_bloc = i.getStringExtra("grau_bloc");

        this.n_intents = ( EditText ) findViewById( R.id.n_intnets );
        this.comentaris = ( EditText ) findViewById(R.id.comentari);

        BlocViewHolder viewHolder = new BlocViewHolder();

        viewHolder.grau_bloc = ( ImageView ) this.findViewById( R.id.grau_bloc );
        viewHolder.grau_bloc.setBackgroundColor( getColorGrau( grau_bloc ) );

        viewHolder.nom_bloc = (TextView) this.findViewById( R.id.nom_bloc );
        viewHolder.nom_bloc.setText( nom_bloc );

        viewHolder.zona = (TextView) this.findViewById( R.id.zona );
        viewHolder.zona.setText( zona );

        if ( AndroidNetworkUtility.isConnected( this ) && actiu.equals( "true" ) )
        {
            findViewById( R.id.encadena ).setOnClickListener( new Button.OnClickListener()
            {
                @Override
                public void onClick( View view )
                {
                    strN_intents = n_intents.getText().toString();
                    strComentaris = comentaris.getText().toString();

                    new EncadenaAsyncTask().execute( "http://37.59.100.31:8080/ranking/encadena/" );

                }
            });

        } else {
            relativeLayout.setVisibility(View.GONE);
        }

    }

    @Override
    protected void onResume()
    {
        super.onResume();

        if ( AndroidNetworkUtility.isConnected( this ) )
        {
            new SociEnAsyncTask( context, pk_bloc ).execute(this);

        }

    }

    @Override
    public boolean onCreateOptionsMenu( Menu menu )
    {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_bloc, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        RelativeLayout post_encadena = ( RelativeLayout ) findViewById( R.id.post_encadena );

        RelativeLayout RLSingleBloc = ( RelativeLayout ) findViewById( R.id.RLSingleBloc );
        RelativeLayout RLComentaris = ( RelativeLayout ) findViewById( R.id.RLComentaris );

        switch (item.getItemId())
        {
            case R.id.veure_comentaris:

                // mostrem comentaris
                if ( CS )
                {
                    if ( AndroidNetworkUtility.isConnected( this ) )
                    {
                        post_encadena.setVisibility( View.GONE );

                        RLSingleBloc.setVisibility( View.GONE );
                        RLComentaris.setVisibility( View.VISIBLE );

                        new ComentarisAsyncTask( this, pk_bloc ).execute( this );

                        CS = ! CS;

                    }
                }

                // mostrem socis
                else {
                    post_encadena.setVisibility( View.VISIBLE );

                    RLSingleBloc.setVisibility( View.VISIBLE );
                    RLComentaris.setVisibility( View.GONE );

                    CS = ! CS;
                }

                return true;

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }

    private class EncadenaAsyncTask extends AsyncTask<String, Void, String>
    {
        @Override
        protected String doInBackground( String... urls )
        {

            POST( urls[0], strN_intents, strComentaris );

            return "EncadenaAsyncTask DONE !";

        }

        // onPostExecute displays the results of the AsyncTask.
        @Override
        protected void onPostExecute( String result )
        {
            onResume();
        }

    }

    public void POST ( String url, String n_intents, String comentaris )
    {
        InputStream inputStream = null;
        String result = "";

        try
        {
            // create HttpClient
            HttpClient httpclient = new DefaultHttpClient();

            // make POST request to the given URL
            HttpPost httpPost = new HttpPost( url );
            httpPost.addHeader("Authorization","Token " +token);

            // build jsonObject
            JSONObject jsonObject = new JSONObject();

            jsonObject.put( "soci", pk_soci );
            jsonObject.put( "bloc", pk_bloc );
            jsonObject.put( "n_intents", n_intents );
            jsonObject.put( "comentaris", comentaris );

            String json = jsonObject.toString();

            // set json to StringEntity
            StringEntity se = new StringEntity( json );

            // set httpPost Entity
            httpPost.setEntity( se );

            // set some headers to inform server about the type of the content
            httpPost.setHeader("Accept", "application/json");
            httpPost.setHeader("Content-type", "application/json");

            // execute POST request to the given URL
            HttpResponse httpResponse = httpclient.execute( httpPost );

            // receive response as inputStream
            inputStream = httpResponse.getEntity().getContent();

           if( inputStream != null )
                Log.d("BlocOnClick", "POST - inputStream OK !" );

            else
                Log.e("BlocOnClick", "POST - inputStream is null" );

        } catch ( Exception e ) {
            Log.d( "BlocOnClick", "POST - error sending encadena json" );

        }
        
    }

    public int getColorGrau( String grau )
    {
        switch ( grau )
        {
            case "R":
                return Color.parseColor("#ad1457");

            case "G":
                return Color.parseColor("#f9a825");

            case "T":
                return Color.parseColor("#ef6c00");

            case "V":
                return Color.parseColor("#c62828");

            case "M":
                return Color.parseColor("#4e342e");

            case "VE":
                return Color.parseColor("#2e7d32");

            case "B":
                return Color.parseColor("#1565c0");

            case "L":
                return Color.parseColor("#6a1b9a");

            case "N":
                return Color.parseColor("#000000");

        }

        return Color.parseColor("#000000");

    }

}
