package ub.mywall;

import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;

import com.google.zxing.integration.android.IntentIntegrator;
import com.google.zxing.integration.android.IntentResult;

import java.util.ArrayList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;


public class MainActivity extends FragmentActivity implements
        ActionBar.TabListener {

    private Context context;

    private ViewPager viewPager;

    private TabsPagerAdapter mAdapter;

    private ActionBar actionBar;

    private String[] tabs = { "Blocs", "Socis", "Favorits" };

    @Override
    protected void onCreate( Bundle savedInstanceState )
    {
        super.onCreate(savedInstanceState);
        setContentView( R.layout.activity_main );

        this.context = this;

        // Initilization
        viewPager = ( ViewPager ) findViewById( R.id.pager );

        actionBar = getActionBar();
        mAdapter = new TabsPagerAdapter( getSupportFragmentManager() );

        // viewPager.setOffscreenPageLimit(3);
        viewPager.setAdapter( mAdapter );

        if ( actionBar != null )
        {
            actionBar.setHomeButtonEnabled(false);
            actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);

            // Adding Tabs
            for (String tab_name : tabs) {
                actionBar.addTab(actionBar.newTab().setText(tab_name)
                        .setTabListener(this));
            }
        }

        /**
         * on swiping the viewpager make respective tab selected
         * */
        viewPager.setOnPageChangeListener(new ViewPager.OnPageChangeListener()
        {
            @Override
            public void onPageSelected(int position)
            {
                // on changing the page
                // make respected tab selected
                actionBar.setSelectedNavigationItem( position );
            }

            @Override
            public void onPageScrolled( int arg0, float arg1, int arg2 ) {  }

            @Override
            public void onPageScrollStateChanged( int arg0 ) {  }

        });

    }

    @Override
    public void onTabReselected( Tab tab, FragmentTransaction ft ) {    }

    @Override
    public void onTabSelected( Tab tab, FragmentTransaction ft ) {
        // on tab selected
        // show respected fragment view
        viewPager.setCurrentItem( tab.getPosition() );
    }

    @Override
    public void onTabUnselected( Tab tab, FragmentTransaction ft ) {    }

    @Override
    public boolean onCreateOptionsMenu( Menu menu )
    {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate( R.menu.menu_main, menu );
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_qr :
                // User chose the "QR" action
                IntentIntegrator integrator = new IntentIntegrator( this );
                integrator.initiateScan();

                return true;

            case R.id.action_profile :
                // User chose the "Profile" action
                startActivity( new Intent( MainActivity.this, Perfil.class ) );

                return true;

            case R.id.action_logout:
                // User chose the "Sortir" action
                SharedPreferences mShared = MainActivity.this.getSharedPreferences( "claus", Context.MODE_PRIVATE );
                SharedPreferences.Editor mEditor = mShared.edit();

                mEditor.remove("pk_soci");
                mEditor.commit();

                mEditor.remove("token");
                mEditor.commit();

                startActivity( new Intent( MainActivity.this, LoginActivity.class ) );

                finish();

                return true;

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }

    public void onActivityResult ( int requestCode, int resultCode, Intent intent )
    {
        IntentResult scanResult = IntentIntegrator.parseActivityResult(requestCode, resultCode, intent);

        if ( scanResult != null )
        {
            String sr = scanResult.getContents();
            Log.i( "MainActivity", "onActivityResult - scanResult: " + sr );

            try
            {
                int int_pk_bloc = Integer.parseInt(sr.substring(38, 43));

                String pk_bloc = String.valueOf(int_pk_bloc);

                ArrayList<Bloc> blocList = null;
                Bloc bloc = null;

                SingleBlocAsyncTask singleBlocAsyncTask = new SingleBlocAsyncTask( this );

                singleBlocAsyncTask.execute( context );

                try {
                    singleBlocAsyncTask.get( 3000, TimeUnit.MILLISECONDS );

                } catch (InterruptedException e) {
                    e.printStackTrace();

                } catch (ExecutionException e) {
                    e.printStackTrace();

                } catch (TimeoutException e) {
                    e.printStackTrace();
                }

                blocList = singleBlocAsyncTask.getBlocList();

                if ( blocList != null )
                {
                    for ( Bloc b : blocList )
                    {
                        if ( b.getPk().equals( pk_bloc ) )
                        {
                            bloc = b;
                            break;
                        }

                    }

                }

                Intent i = new Intent( context, BlocOnClick.class );

                // sending data to new activity
                i.putExtra("pk_bloc", pk_bloc);
                i.putExtra("nom_bloc", bloc.getNom());
                i.putExtra("zona", bloc.getZona());
                i.putExtra("actiu", bloc.getActiu());
                i.putExtra("grau_bloc", bloc.getGrau());

                context.startActivity(i);

            } catch ( Exception e ) {
                Log.e( "MainActivity", "onActivityResult - QR scann error !!!" );

            }

        }

        // else continue with any other code you need in the method

    }

}
